package hamster;

/**
 * Die Klasse realisiert einen FrissHamster, der die Gewinnstrategie beim
 * NimmSpiel kennt.
 * 
 * @author Dietrich Boles (Universitaet Oldenburg)
 * @version 1.0 (28.01.2006)
 * 
 */
public class SchlauerSpielHamster extends SpielHamster {

	/**
	 * Konstruktor zum Erzeugen und Initialisieren eines Hamsters mit den
	 * uebergebenen Parametern
	 * 
	 * @param reihe
	 *            die Reihe des Territoriums, in der der Hamster erzeugt wird
	 * @param spalte
	 *            die Spalte des Territoriums, in der der Hamster erzeugt wird
	 * @param blickrichtung
	 *            die Richtung, in der der Hamster anfangs schaut (siehe
	 *            Konstanten)
	 * @param anzahlKoerner
	 *            die Anzahl an Koernern, die der Hamster anfangs im Maul hat
	 */
	public SchlauerSpielHamster(int reihe, int spalte, int blickrichtung,
			int koernerAnzahl) {
		super(reihe, spalte, blickrichtung, koernerAnzahl);
	}

	/**
	 * die Methode zum Fressen; uebergeben wird, wie viele Koerner der Gegner
	 * gefressen hat; geliefert wird, wie viele Koerner der Hamster gefressen
	 * hat
	 * 
	 * @param gegnerAnzahl
	 *            Anzahl der vom Gegner zuvor gefressenen Koerner
	 * @return Anzahl selbst gefressener Koerner
	 */
	public int naechsterSpielzug(int gegnerAnzahl) {
		this.laengeDerKornReihe = this.laengeDerKornReihe - gegnerAnzahl;

		// laufe wieder bis zum Anfang der Koernerreihe
		this.vor(gegnerAnzahl);

		// Gewinnstrategie: wenn moeglich, immer eine durch
		// 3 teilbare Anzahl an Koernern uebriglassen
		if (this.laengeDerKornReihe % 3 != 0) {
			return this.vorUndNimm(this.laengeDerKornReihe % 3);
		} else { // Gewinnstrategie nicht anwendbar
			return this.vorUndNimm(1);
		}
	}

	private int vorUndNimm(int anzahl) {
		int schritte = 0;
		while (this.vornFrei() && anzahl > 0) {
			this.vor();
			this.nimm();
			schritte = schritte + 1;
			anzahl = anzahl - 1;
		}
		return schritte;
	}
}
